/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-16 17:49:59.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service;

import cn.com.qmth.examcloud.app.exception.ApiException;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.utils.DateUtils;
import cn.com.qmth.examcloud.app.utils.HttpUtils;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.apache.commons.io.FilenameUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import static cn.com.qmth.examcloud.app.model.Constants.*;

/**
 * 网考业务服务接口
 */
@Service
public class NetExamService {
    private static Logger log = LoggerFactory.getLogger(NetExamService.class);
    @Autowired
    private PropertyService propertyService;

    public Result getCurrentTime(String key, String token) throws Exception {
        final String requestUrl = propertyService.getNetExamUrl() + "/api/online_exam_course/currentTime";
        //封装请求参数
        Request request = new Request.Builder()
                .url(requestUrl)
                .get()
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .build();
        //执行请求
        Result<String> result = HttpUtils.call(request);
        if (result.isSuccess()) {
            //转换日期格式
            result.setData(DateUtils.format(result.getData()));
        }
        return result;
    }

    public Result getOfflineExamCourseList(String key, String token) throws Exception {
        final String requestUrl = propertyService.getNetExamUrl() + "/api/offline_exam/getOfflineCourse";
        //封装请求参数
        Request request = new Request.Builder()
                .url(requestUrl)
                .get()
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .build();
        //执行请求
        return HttpUtils.call(request);
    }

    public Result startOfflineExamRecord(String key, String token, String examStudentId) throws Exception {
        final String requestUrl = String.format(propertyService.getNetExamUrl() + "/api/offline_exam/start?examStudentId=%s", examStudentId);
        //封装请求参数
        Request request = new Request.Builder()
                .url(requestUrl)
                .get()
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .build();
        //执行请求
        return HttpUtils.call(request);
    }

    public Result uploadPaperAnswer(String key, String token, String examRecordId, byte[] fileBytes, String fileName, String md5) throws Exception {
        Assert.notNull(examRecordId, "FileName must not be null.");
        Assert.notNull(fileName, "ExamRecordId must not be null.");
        String fileType = FilenameUtils.getExtension(fileName.toLowerCase());
        if (!TYPE_ZIP.equals(fileType) && !TYPE_PDF.equals(fileType)) {
            throw new ApiException("FileType must be zip or pdf.");
        }
        if (fileBytes.length == 0) {
            throw new ApiException("File must be not empty.");
        }
        final String requestUrl = String.format(propertyService.getNetExamUrl() + "/api/offline_exam/%s/submit", examRecordId);
        MultipartBody.Builder formBody = new MultipartBody.Builder().setType(MultipartBody.FORM);
        RequestBody body = RequestBody.create(MediaType.parse("application/octet-stream"), fileBytes);
        formBody.addFormDataPart("file", fileName, body);
        formBody.addFormDataPart("fileType", fileType);
        formBody.addFormDataPart("md5", md5);
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody.build())
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .build();
        //执行请求
        return HttpUtils.call(request);
    }

    public Result getPracticeExamCourseList(String key, String token, String examId) throws Exception {
        return null;
    }

    public Result getExamRecordHeartbeat(String key, String token, String examRecordId) throws Exception {
        return null;
    }

    public Result startPracticeExamRecord(String key, String token, String examStudentId) throws Exception {
        return null;
    }

    public Result getExamRecordPaperStructList(String key, String token, String examRecordId) throws Exception {
        return null;
    }

    public Result getExamRecordPaperQuestionList(String key, String token, String examRecordId) throws Exception {
        return null;
    }

    public Result getExamRecordPaperQuestionDetail(String key, String token, String questionId) throws Exception {
        return null;
    }

    public Result updateExamRecordPaperQuestionAnswer(String key, String token, String examQuestionId, String studentAnswer) throws Exception {
        return null;
    }

    public Result submitPracticeExamRecord(String key, String token, String examRecordId) throws Exception {
        return null;
    }

    public Result checkOnlineExamRecord(String key, String token) throws Exception {
        return null;
    }

    public Result getExamRecordPracticeTotalInfo(String key, String token, String examStudentId) throws Exception {
        return null;
    }

    public Result getExamRecordPracticeHistoryList(String key, String token, String examStudentId) throws Exception {
        return null;
    }

    public Result getExamRecordTotalInfo(String key, String token, String examRecordId) throws Exception {
        return null;
    }

    public Result getExamRecordQuestionDetailList(String key, String token, String examRecordId) throws Exception {
        return null;
    }

    public Result getExamRecordQuestionAudioPlayTimes(String key, String token, String questionId) throws Exception {
        return null;
    }

    public Result updateExamRecordQuestionAudioPlayTimes(String key, String token, String questionId, String mediaName) throws Exception {
        return null;
    }

}